@@ -38,10 +38,8 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#if GTK_CHECK_VERSION (3, 0, 0)
-#include <gdk/gdkkeysyms-compat.h>
-#endif
 #include <gio/gio.h>
+#include <libmate-desktop/mate-aboutdialog.h>
 
 #include <mate-panel-applet.h>
 #include <mate-panel-applet-gsettings.h>
@@ -571,7 +569,7 @@ static void display_about_dialog(GtkAction* action, FishApplet* fish)
 
 	descr = g_strdup_printf(about_format, fish->name);
 
-	gtk_show_about_dialog(NULL,
+	mate_show_about_dialog(NULL,
 		"program-name", _("Fish"),
 		"authors", authors,
 		"comments", descr,
@@ -849,10 +847,6 @@ static void display_fortune_dialog(FishApplet* fish)
 
 		gtk_window_set_icon_name (GTK_WINDOW (fish->fortune_dialog),
 					  FISH_ICON);
-#if !GTK_CHECK_VERSION (3, 0, 0)
-		gtk_dialog_set_has_separator (
-			GTK_DIALOG (fish->fortune_dialog), FALSE);
-#endif
 
 		gtk_dialog_set_default_response (
 			GTK_DIALOG (fish->fortune_dialog), GTK_RESPONSE_CLOSE);
@@ -1135,8 +1129,8 @@ static char* get_location(void)
 			count++;
 
 	if (count != 2) {
-		return NULL;
 		g_free (buffer);
+		return NULL;
 	}
 
 	memcpy (location, &buffer [i + 2], len - i - 2);
@@ -1622,12 +1616,12 @@ static void change_water(FishApplet* fish)
 static gboolean handle_keypress(GtkWidget* widget, GdkEventKey* event, FishApplet* fish)
 {
 	switch (event->keyval) {
-	case GDK_space:
-	case GDK_KP_Space:
-	case GDK_Return:
-	case GDK_KP_Enter:
-	case GDK_ISO_Enter:
-	case GDK_3270_Enter:
+	case GDK_KEY_space:
+	case GDK_KEY_KP_Space:
+	case GDK_KEY_Return:
+	case GDK_KEY_KP_Enter:
+	case GDK_KEY_ISO_Enter:
+	case GDK_KEY_3270_Enter:
 		if (fish->april_fools) {
 			change_water (fish);
 			return TRUE;
