@@ -34,6 +34,11 @@
 #define ICON_SPACING 1
 #define MIN_BOX_SIZE 3
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gtk_vbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_VERTICAL, Y)
+#define gtk_hbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, Y)
+#endif
+
 typedef struct
 {
   NaTrayManager *tray_manager;
@@ -84,6 +89,7 @@ static TraysScreen *trays_screens = NULL;
 
 static void icon_tip_show_next (IconTip *icontip);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 /* NaBox, an instantiable GtkBox */
 
 typedef GtkBox      NaBox;
@@ -102,6 +108,7 @@ static void
 na_box_class_init (NaBoxClass *klass)
 {
 }
+#endif
 
 /* NaTray */
 
@@ -247,13 +254,17 @@ tray_removed (NaTrayManager *manager,
               TraysScreen   *trays_screen)
 {
   NaTray *tray;
+  NaTrayPrivate *priv;
 
   tray = g_hash_table_lookup (trays_screen->icon_table, icon);
   if (tray == NULL)
     return;
 
+  priv = tray->priv;
   g_assert (tray->priv->trays_screen == trays_screen);
 
+  gtk_container_remove (GTK_CONTAINER (priv->box), icon);
+
   g_hash_table_remove (trays_screen->icon_table, icon);
   /* this will also destroy the tip associated to this icon */
   g_hash_table_remove (trays_screen->tip_table, icon);
@@ -600,15 +611,16 @@ na_tray_init (NaTray *tray)
   gtk_container_add (GTK_CONTAINER (tray), priv->frame);
   gtk_widget_show (priv->frame);
 
-  priv->box = g_object_new (na_box_get_type (), NULL);
 #if GTK_CHECK_VERSION (3, 0, 0)
+  priv->box = gtk_box_new (priv->orientation, ICON_SPACING);
   g_signal_connect (priv->box, "draw",
                     G_CALLBACK (na_tray_draw_box), NULL);
 #else
+  priv->box = g_object_new (na_box_get_type (), NULL);
   g_signal_connect (priv->box, "expose-event",
                     G_CALLBACK (na_tray_expose_box), tray);
-#endif
   gtk_box_set_spacing (GTK_BOX (priv->box), ICON_SPACING);
+#endif
   gtk_container_add (GTK_CONTAINER (priv->frame), priv->box);
   gtk_widget_show (priv->box);
 }
