@@ -38,6 +38,8 @@
 #include <gdk/gdkkeysyms-compat.h>
 #endif
 
+#include <libmate-desktop/mate-aboutdialog.h>
+
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
 
@@ -81,7 +83,7 @@ static void window_menu_about(GtkAction* action, WindowMenu* window_menu)
 		"Copyright \xc2\xa9 2001 Free Software Foundation, Inc.\n"
 		"Copyright \xc2\xa9 2000 Helix Code, Inc.";
 
-	gtk_show_about_dialog(GTK_WINDOW(window_menu->applet),
+	mate_show_about_dialog(GTK_WINDOW(window_menu->applet),
 		"program-name", _("Window Selector"),
 		"authors", authors,
 		"comments", _("The Window Selector shows a list of all windows in a menu and lets you browse them."),
@@ -120,39 +122,71 @@ static void window_menu_destroy(GtkWidget* widget, WindowMenu* window_menu)
 }
 
 #if GTK_CHECK_VERSION (3, 0, 0)
-static gboolean window_menu_on_draw(GtkWidget* widget, cairo_t* cr, gpointer data)
+static gboolean window_menu_on_draw (GtkWidget* widget,
+				     cairo_t*   cr,
+				     gpointer   data)
+{
+	GtkStyleContext *context;
+	GtkStateFlags    state;
+	WindowMenu      *window_menu = data;
+
+	if (!gtk_widget_has_focus (window_menu->applet))
+		return FALSE;
+
+	state = gtk_widget_get_state_flags (widget);
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_save (context);
+	gtk_style_context_set_state (context, state);
+
+	cairo_save (cr);
+	gtk_render_focus (context, cr,
+			  0., 0.,
+			  gtk_widget_get_allocated_width (widget),
+			  gtk_widget_get_allocated_height (widget));
+			  cairo_restore (cr);
+
+	gtk_style_context_restore (context);
+
+	return FALSE;
+}
 #else
-static gboolean window_menu_on_expose(GtkWidget* widget, GdkEventExpose* event, gpointer data)
-#endif
+static gboolean window_menu_on_expose (GtkWidget*      widget,
+				       GdkEventExpose* event,
+				       gpointer        data)
 {
 	WindowMenu* window_menu = data;
 
 	if (gtk_widget_has_focus(window_menu->applet))
 		gtk_paint_focus(gtk_widget_get_style(widget),
-#if GTK_CHECK_VERSION (3, 0, 0)
-						cr,
-#else
 						gtk_widget_get_window(widget),
-#endif
 						gtk_widget_get_state(widget),
-#if !GTK_CHECK_VERSION (3, 0, 0)
 						NULL,
-#endif
 						widget,
 						"menu-applet",
 						0, 0,
-#if GTK_CHECK_VERSION (3, 0, 0)
-						gtk_widget_get_allocated_width (widget),
-						gtk_widget_get_allocated_height (widget));
-#else
 						-1, -1);
-#endif
 
 	return FALSE;
 }
+#endif
 
 static inline void force_no_focus_padding(GtkWidget* widget)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkCssProvider *provider;
+
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (provider,
+					 "#PanelApplet-window-menu-applet-button {\n"
+					 " border-width: 0px;\n"
+					 " -GtkWidget-focus-line-width: 0px;\n"
+					 " -GtkWidget-focus-padding: 0px; }",
+					 -1, NULL);
+	gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	g_object_unref (provider);
+#else
 	gboolean first_time = TRUE;
 
 	if (first_time)
@@ -168,6 +202,7 @@ static inline void force_no_focus_padding(GtkWidget* widget)
 			"\n");
 		first_time = FALSE;
 	}
+#endif
 
 	gtk_widget_set_name(widget, "PanelApplet-window-menu-applet-button");
 }
