@@ -135,11 +135,16 @@ mate_panel_applet_frame_draw (GtkWidget *widget,
 		cairo_pattern_destroy (bg_pattern);
 	}
 
+	cairo_rectangle (cr,
+		frame->priv->handle_rect.x,
+		frame->priv->handle_rect.y,
+		frame->priv->handle_rect.width,
+		frame->priv->handle_rect.height);
+	cairo_clip (cr);
 	gtk_render_handle (context, cr,
-			   frame->priv->handle_rect.x,
-			   frame->priv->handle_rect.y,
-			   frame->priv->handle_rect.width,
-			   frame->priv->handle_rect.height);
+			   0, 0,
+			   gtk_widget_get_allocated_width (widget),
+			   gtk_widget_get_allocated_height (widget));
 
 	cairo_restore (cr);
 
@@ -467,10 +472,15 @@ button_event_in_rect (GdkEventButton *event,
 
 static gboolean
 mate_panel_applet_frame_button_changed (GtkWidget      *widget,
-				   GdkEventButton *event)
+					GdkEventButton *event)
 {
 	MatePanelAppletFrame *frame;
-	gboolean          handled = FALSE;
+	gboolean              handled = FALSE;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkDisplay *display;
+	GdkDevice *pointer;
+	GdkDeviceManager *device_manager;
+#endif
 
 	frame = MATE_PANEL_APPLET_FRAME (widget);
 
@@ -499,6 +509,12 @@ mate_panel_applet_frame_button_changed (GtkWidget      *widget,
 	case 3:
 		if (event->type == GDK_BUTTON_PRESS ||
 		    event->type == GDK_2BUTTON_PRESS) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+			display = gtk_widget_get_display (widget);
+			device_manager = gdk_display_get_device_manager (display);
+			pointer = gdk_device_manager_get_client_pointer (device_manager);
+			gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
+#endif
 			gdk_pointer_ungrab (GDK_CURRENT_TIME);
 
 			MATE_PANEL_APPLET_FRAME_GET_CLASS (frame)->popup_menu (frame,
@@ -624,6 +640,7 @@ mate_panel_applet_frame_change_background (MatePanelAppletFrame    *frame,
 
 	g_return_if_fail (PANEL_IS_WIDGET (parent));
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	if (frame->priv->has_handle) {
 		PanelBackground *background;
 
@@ -631,6 +648,7 @@ mate_panel_applet_frame_change_background (MatePanelAppletFrame    *frame,
 		panel_background_change_background_on_widget (background,
 							      GTK_WIDGET (frame));
 	}
+#endif
 
 	MATE_PANEL_APPLET_FRAME_GET_CLASS (frame)->change_background (frame, type);
 }
@@ -949,7 +967,7 @@ mate_panel_applet_frame_activating_free (MatePanelAppletFrameActivating *frame_a
 GdkScreen *
 panel_applet_frame_activating_get_screen (MatePanelAppletFrameActivating *frame_act)
 {
-	return gtk_widget_get_screen (frame_act->panel);
+    return gtk_widget_get_screen (GTK_WIDGET(frame_act->panel));
 }
 
 PanelOrientation
