@@ -50,6 +50,12 @@
 #include "panel-addto.h"
 #include "panel-icon-names.h"
 #include "panel-schemas.h"
+#include "panel-stock-icons.h"
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gtk_vbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_VERTICAL, Y)
+#define gtk_hbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, Y)
+#endif
 
 typedef struct {
 	PanelWidget *panel_widget;
@@ -183,7 +189,7 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 };
 
 enum {
-	COLUMN_ICON,
+	COLUMN_ICON_NAME,
 	COLUMN_TEXT,
 	COLUMN_DATA,
 	COLUMN_SEARCH,
@@ -266,18 +272,6 @@ panel_addto_make_text (const char *name,
 	return result;
 }
 
-#define ICON_SIZE 32
-
-static GdkPixbuf *
-panel_addto_make_pixbuf (const char *filename)
-{
-	//FIXME: size shouldn't be fixed but should depend on the font size
-	return panel_load_icon (gtk_icon_theme_get_default (),
-				filename,
-				ICON_SIZE, ICON_SIZE, ICON_SIZE,
-				NULL);
-}
-
 static void
 panel_addto_drag_data_get_cb (GtkWidget        *widget,
 			      GdkDragContext   *context,
@@ -301,7 +295,7 @@ panel_addto_drag_begin_cb (GtkWidget      *widget,
 	GtkTreePath  *path;
 	GtkTreeIter   iter;
 	GtkTreeIter   filter_iter;
-	GdkPixbuf    *pixbuf;
+	gchar         *icon_name;
 
 	filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 
@@ -313,11 +307,11 @@ panel_addto_drag_begin_cb (GtkWidget      *widget,
 
 	child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
 	gtk_tree_model_get (child_model, &iter,
-	                    COLUMN_ICON, &pixbuf,
+	                    COLUMN_ICON_NAME, &icon_name,
 	                    -1);
 
-	gtk_drag_set_icon_pixbuf (context, pixbuf, 0, 0);
-	g_object_unref (pixbuf);
+	gtk_drag_set_icon_name (context, icon_name, 0, 0);
+	g_free (icon_name);
 }
 
 static void
@@ -426,39 +420,29 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 			 PanelAddtoItemInfo *applet)
 {
 	char *text;
-	GdkPixbuf *pixbuf;
 	GtkTreeIter iter;
 
 	if (applet == NULL) {
 		gtk_list_store_append (model, &iter);
 		gtk_list_store_set (model, &iter,
-				    COLUMN_ICON, NULL,
+				    COLUMN_ICON_NAME, NULL,
 				    COLUMN_TEXT, NULL,
 				    COLUMN_DATA, NULL,
 				    COLUMN_SEARCH, NULL,
 				    -1);
 	} else {
-		pixbuf = NULL;
-
-		if (applet->icon != NULL) {
-			pixbuf = panel_addto_make_pixbuf (applet->icon);
-		}
-
 		gtk_list_store_append (model, &iter);
 
 		text = panel_addto_make_text (applet->name,
 					      applet->description);
 
 		gtk_list_store_set (model, &iter,
-				    COLUMN_ICON, pixbuf,
+				    COLUMN_ICON_NAME, applet->icon,
 				    COLUMN_TEXT, text,
 				    COLUMN_DATA, applet,
 				    COLUMN_SEARCH, applet->name,
 				    -1);
 
-		if (pixbuf)
-			g_object_unref (pixbuf);
-
 		g_free (text);
 	}
 }
@@ -504,7 +488,7 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 					    (GCompareFunc) panel_addto_applet_info_sort_func);
 
 	model = gtk_list_store_new (NUMBER_COLUMNS,
-				    GDK_TYPE_PIXBUF,
+				    G_TYPE_STRING,
 				    G_TYPE_STRING,
 				    G_TYPE_POINTER,
 				    G_TYPE_STRING);
@@ -652,7 +636,6 @@ panel_addto_populate_application_model (GtkTreeStore *store,
 	PanelAddtoAppList *data;
 	GtkTreeIter        iter;
 	char              *text;
-	GdkPixbuf         *pixbuf;
 	GSList            *app;
 
 	for (app = app_list; app != NULL; app = app->next) {
@@ -661,17 +644,13 @@ panel_addto_populate_application_model (GtkTreeStore *store,
 
 		text = panel_addto_make_text (data->item_info.name,
 					      data->item_info.description);
-		pixbuf = panel_addto_make_pixbuf (data->item_info.icon);
 		gtk_tree_store_set (store, &iter,
-				    COLUMN_ICON, pixbuf,
+				    COLUMN_ICON_NAME, data->item_info.icon,
 				    COLUMN_TEXT, text,
 				    COLUMN_DATA, &(data->item_info),
 				    COLUMN_SEARCH, data->item_info.name,
 				    -1);
 
-		if (pixbuf)
-			g_object_unref (pixbuf);
-
 		g_free (text);
 
 		if (data->children != NULL)
@@ -690,7 +669,7 @@ static void panel_addto_make_application_model(PanelAddtoDialog* dialog)
 	if (dialog->filter_application_model != NULL)
 		return;
 
-	store = gtk_tree_store_new(NUMBER_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
+	store = gtk_tree_store_new(NUMBER_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
 
 	tree = matemenu_tree_lookup("mate-applications.menu", MATEMENU_TREE_FLAGS_NONE);
 	matemenu_tree_set_sort_key(tree, MATEMENU_TREE_SORT_DISPLAY_NAME);
@@ -713,7 +692,7 @@ static void panel_addto_make_application_model(PanelAddtoDialog* dialog)
 		GtkTreeIter iter;
 
 		gtk_tree_store_append(store, &iter, NULL);
-		gtk_tree_store_set(store, &iter, COLUMN_ICON, NULL, COLUMN_TEXT, NULL, COLUMN_DATA, NULL, COLUMN_SEARCH, NULL, -1);
+		gtk_tree_store_set(store, &iter, COLUMN_ICON_NAME, NULL, COLUMN_TEXT, NULL, COLUMN_DATA, NULL, COLUMN_SEARCH, NULL, -1);
 
 		panel_addto_make_application_list(&dialog->settings_list, root, "mate-settings.menu");
 		panel_addto_populate_application_model(store, NULL, dialog->settings_list);
@@ -1225,9 +1204,6 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 {
 	PanelAddtoDialog *dialog;
 	GtkWidget *dialog_vbox;
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	GtkWidget *vbox;
-#endif
 	GtkWidget *inner_vbox;
 	GtkWidget *find_hbox;
 	GtkWidget *sw;
@@ -1262,40 +1238,23 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 			       GTK_STOCK_CLOSE,
 			       GTK_RESPONSE_CLOSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->addto_dialog),
-				      FALSE);
-#endif
+
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->addto_dialog),
 					 PANEL_ADDTO_RESPONSE_ADD);
 
 	gtk_container_set_border_width (GTK_CONTAINER (dialog->addto_dialog), 5);
 
 	dialog_vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog->addto_dialog));
-#if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_box_set_spacing (GTK_BOX (dialog_vbox), 12);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), 5);
-#else
-	gtk_box_set_spacing (GTK_BOX (dialog_vbox), 2);
-#endif
 
 	g_signal_connect (G_OBJECT (dialog->addto_dialog), "response",
 			  G_CALLBACK (panel_addto_dialog_response), dialog);
 	g_signal_connect (dialog->addto_dialog, "destroy",
 			  G_CALLBACK (panel_addto_dialog_destroy), dialog);
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	vbox = gtk_vbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-	gtk_container_add (GTK_CONTAINER (dialog_vbox), vbox);
-#endif
-
 	inner_vbox = gtk_vbox_new (FALSE, 6);
-#if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), inner_vbox, TRUE, TRUE, 0);
-#else
-	gtk_box_pack_start (GTK_BOX (vbox), inner_vbox, TRUE, TRUE, 0);
-#endif
 
 	find_hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (inner_vbox), find_hbox, FALSE, FALSE, 0);
@@ -1335,12 +1294,13 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	renderer = g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
 				 "xpad", 4,
 				 "ypad", 4,
+				 "stock-size", panel_add_to_icon_get_size(),
 				 NULL);
 
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->tree_view),
 						     -1, NULL,
 						     renderer,
-						     "pixbuf", COLUMN_ICON,
+						     "icon_name", COLUMN_ICON_NAME,
 						     NULL);
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -1377,11 +1337,7 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	gtk_container_add (GTK_CONTAINER (sw), dialog->tree_view);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_widget_show_all (dialog_vbox);
-#else
-	gtk_widget_show_all (vbox);
-#endif
 
 	panel_toplevel_push_autohide_disabler (dialog->panel_widget->toplevel);
 	panel_widget_register_open_dialog (panel_widget,
