@@ -54,11 +54,10 @@ struct _PanelBackground {
 
 	GtkOrientation          orientation;
 	GdkRectangle            region;
+	GdkPixbuf              *transformed_image;
 #if GTK_CHECK_VERSION (3, 0, 0)
-	cairo_pattern_t        *transformed_pattern;
 	cairo_pattern_t        *composited_pattern;
 #else
-	GdkPixbuf              *transformed_image;
 	GdkPixbuf              *composited_image;
 #endif
 
@@ -159,7 +158,12 @@ const GdkPixmap     *panel_background_get_pixmap (PanelBackground *background);
 PanelBackgroundType
       panel_background_effective_type    (PanelBackground     *background);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 void panel_background_change_background_on_widget (PanelBackground *background,
 						   GtkWidget       *widget);
 
+#else
+void panel_background_apply_css(GtkWidget* widget);
+#endif
+
 #endif /* __PANEL_BACKGROUND_H__ */
