@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
 
  * Authors:
  *	Frank Worsley <fworsley@shaw.ca>
@@ -34,6 +34,8 @@ extern "C" {
 void panel_run_dialog_present           (GdkScreen  *screen,
 					 guint32    activate_time);
 
+void panel_run_dialog_quit_on_destroy (void);
+
 #ifdef __cplusplus
 }
 #endif
