@@ -29,10 +29,8 @@
 #include <gio/gio.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #include <libmate-desktop/mate-desktop-utils.h>
-#endif
 #include <libmate-desktop/mate-gsettings.h>
 
 #include <libpanel-util/panel-error.h>
@@ -116,7 +114,11 @@ panel_push_window_busy (GtkWidget *window)
 		if (win != NULL) {
 			GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
 			gdk_window_set_cursor (win, cursor);
+#if GTK_CHECK_VERSION (3, 0, 0)
+			g_object_unref (cursor);
+#else
 			gdk_cursor_unref (cursor);
+#endif
 			gdk_flush ();
 		}
 	}
@@ -256,6 +258,33 @@ panel_uri_exists (const char *uri)
 	return ret;
 }
 
+GIcon *
+panel_gicon_from_icon_name (const char *icon_name) {
+	GIcon *icon = NULL;
+
+	if (icon_name == NULL) {
+		return NULL;
+	}
+
+	if (g_path_is_absolute(icon_name)) {
+		if (!g_file_test (icon_name, G_FILE_TEST_EXISTS)) {
+			gchar *name = g_path_get_basename (icon_name);
+			icon = g_themed_icon_new (name);
+			g_free (name);
+		} else {
+			GFile *gfile = g_file_new_for_path (icon_name);
+			icon = g_file_icon_new (gfile);
+			g_object_unref (gfile);
+		}
+	}
+	else {
+		gchar *name = panel_xdg_icon_remove_extension (icon_name);
+		icon = g_themed_icon_new (name);
+		g_free (name);
+	}
+	return icon;
+}
+
 char *
 panel_find_icon (GtkIconTheme  *icon_theme,
 		 const char    *icon_name,
@@ -434,11 +463,7 @@ void panel_lock_screen_action(GdkScreen* screen, const char* action)
 		return;
 	}
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (!mate_gdk_spawn_command_line_on_screen(screen, command, &error))
-#else
-	if (!gdk_spawn_command_line_on_screen(screen, command, &error))
-#endif
 	{
 		char* primary = g_strdup_printf(_("Could not execute '%s'"), command);
 		panel_error_dialog (NULL, screen, "cannot_exec_screensaver", TRUE, primary, error->message);
@@ -457,11 +482,7 @@ void panel_lock_screen(GdkScreen* screen)
 
 static char* panel_launcher_get_personal_path(void)
 {
-	#if GLIB_CHECK_VERSION(2, 6, 0)
 		return g_build_filename(g_get_user_config_dir(), "mate", "panel2.d", "default", "launchers", NULL);
-	#else // glib version < 2.6.0
-		return g_build_filename(g_get_home_dir(), ".config", "mate", "panel2.d", "default", "launchers", NULL);
-	#endif
 }
 
 gboolean
@@ -763,6 +784,7 @@ guess_icon_from_exec (GtkIconTheme *icon_theme,
 		return NULL;
 	}
 
+	g_free (path);
 	return icon_name;
 }
 
