@@ -1662,6 +1662,8 @@ panel_widget_style_set (GtkWidget *widget, GtkStyle  *previous_style)
 #if GTK_CHECK_VERSION (3, 0, 0)
 		context = gtk_widget_get_style_context (widget);
 		state = gtk_widget_get_state_flags (widget);
+		gtk_style_context_add_class(context,"gnome-panel-menu-bar");
+		gtk_style_context_add_class(context,"mate-panel-menu-bar");
 
 		gtk_style_context_get_background_color (context, state, &bg_color);
 		gtk_style_context_get (context, state, "background-image", &bg_image, NULL);
@@ -1694,8 +1696,8 @@ panel_widget_state_changed (GtkWidget    *widget,
 #endif
 {
 #if GTK_CHECK_VERSION (3, 0, 0)
-	panel_widget_set_background_default_style (widget);
 	GTK_WIDGET_CLASS (panel_widget_parent_class)->style_updated (widget);
+	panel_widget_set_background_default_style (widget);
 #else
 	GtkStyle     *style;
 	GtkStateType  state;
@@ -1749,11 +1751,11 @@ panel_widget_realize (GtkWidget *widget)
 #if !GTK_CHECK_VERSION (3, 0, 0)
 	style = gtk_widget_get_style (widget);
 	state = gtk_widget_get_state (widget);
-#endif
 
 	/* For auto-hidden panels with a colored background, we need native
 	 * windows to avoid some uglyness on unhide */
 	gdk_window_ensure_native (window);
+#endif
 
 #if GTK_CHECK_VERSION (3, 0, 0)
 	panel_widget_set_background_default_style (widget);
@@ -2023,14 +2025,30 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 	if (window) {
 		GdkGrabStatus  status;
 		GdkCursor     *fleur_cursor;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkDisplay *display;
+		GdkDevice *pointer;
+		GdkDeviceManager *device_manager;
 
 		fleur_cursor = gdk_cursor_new (GDK_FLEUR);
 
+		display = gdk_window_get_display (window);
+		device_manager = gdk_display_get_device_manager (display);
+		pointer = gdk_device_manager_get_client_pointer (device_manager);
+		status = gdk_device_grab (pointer, window,
+					  GDK_OWNERSHIP_NONE, FALSE,
+					  APPLET_EVENT_MASK,
+					  fleur_cursor, time_);
+
+		g_object_unref (fleur_cursor);
+#else
+		fleur_cursor = gdk_cursor_new (GDK_FLEUR);
+
 		status = gdk_pointer_grab (window, FALSE,
 					   APPLET_EVENT_MASK, NULL,
 					   fleur_cursor, time_);
-
 		gdk_cursor_unref (fleur_cursor);
+#endif
 		gdk_flush ();
 
 		if (status != GDK_GRAB_SUCCESS) {
@@ -2044,11 +2062,25 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 void
 panel_widget_applet_drag_end (PanelWidget *panel)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkDisplay *display;
+	GdkDevice *pointer;
+	GdkDeviceManager *device_manager;
+#endif
+
 	g_return_if_fail (PANEL_IS_WIDGET (panel));
 
 	if (panel->currently_dragged_applet == NULL)
 		return;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	display = gtk_widget_get_display (GTK_WIDGET (panel));
+	device_manager = gdk_display_get_device_manager (display);
+	pointer = gdk_device_manager_get_client_pointer (device_manager);
+
+	gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
+#else
 	gdk_pointer_ungrab (GDK_CURRENT_TIME);
+#endif
 	gtk_grab_remove (panel->currently_dragged_applet->applet);
 	panel_widget_applet_drag_end_no_grab (panel);
 	panel_toplevel_pop_autohide_disabler (panel->toplevel);
