@@ -351,11 +351,14 @@ panel_popup_menu (PanelToplevel *toplevel,
 	panel_data   = g_object_get_data (G_OBJECT (toplevel), "PanelData");
 
 	current_event = gtk_get_current_event ();
-	if (current_event->type == GDK_BUTTON_PRESS)
+	if (current_event && current_event->type == GDK_BUTTON_PRESS)
 		panel_data->insertion_pos = panel_widget_get_cursorloc (panel_widget);
 	else
 		panel_data->insertion_pos = -1;
-	
+
+	if (current_event)
+		gdk_event_free (current_event);
+
 	menu = make_popup_panel_menu (panel_widget);
 	if (!menu)
 		return FALSE;
@@ -1362,7 +1365,7 @@ panel_deletion_dialog (PanelToplevel *toplevel)
 				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
 				NULL);
 
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 				gtk_window_get_screen (GTK_WINDOW (toplevel)));
